//--------------------------------------------------------------
// File     : stm32_ub_syscheck.h
//--------------------------------------------------------------

//--------------------------------------------------------------
#ifndef __STM32F4_UB_SYSCHECK_H
#define __STM32F4_UB_SYSCHECK_H


//--------------------------------------------------------------
// Includes
//--------------------------------------------------------------
#include "stm32f4xx.h"
#include "stm32f4xx_gpio.h"
#include "stm32f4xx_rcc.h"
#include "stm32f4xx_conf.h"




//--------------------------------------------------------------
// Aktivieren, Deaktivieren von einer Funktion
// 1=aktiv, 0=inaktiv
//--------------------------------------------------------------
#define   SYSCHECK_USE_CLOCK    1   // System-Clock
#define   SYSCHECK_USE_ASSERT   1   // ASSERT-Mode
#define   SYSCHECK_USE_MCO      0   // MCO-Clock



//--------------------------------------------------------------
// Fehlernummern
//--------------------------------------------------------------
typedef enum {
  SYSCHECK_OK =0, // keine Fehler
  CLKSOURCE_ERR,  // Clock-Source stimmt nicht
  HCLK_ERR,       // Frq von HCLK stimmt nicht
  PCLK1_ERR,      // Frq von PCLK1 stimmt nicht
  PCLK2_ERR,      // Frq von PCLK2 stimmt nicht
  SYSCLK_ERR,     // Frq von SYSCLK stimmt nicht
  ASSERT_ERR      // Assert Fehler
}SYSCHECK_t;


//--------------------------------------------------------------
// Sollwerte fr "UB_SysCheck_Clock"
//--------------------------------------------------------------
#define  SYSCHECK_CLKSOURCE          8   // Syslock = PLL
#define  SYSCHECK_HCLK_FRQ    168000000  // HCLK    = 168 MHz
#define  SYSCHECK_PCLK1_FRQ    42000000  // PCLK1   =  42 MHz
#define  SYSCHECK_PCLK2_FRQ    84000000  // PCLK2   =  84 MHz
#define  SYSCHECK_SYSCLK_FRQ  168000000  // SysCLK  = 168 MHz



//--------------------------------------------------------------
// Enums und Strukturen fr "UB_SysCheck_Assert"
//--------------------------------------------------------------
#if  SYSCHECK_USE_ASSERT==1

  #ifndef  USE_FULL_ASSERT
    #error  "Im File : stm32f4xx_conf.h muss der Define USE_FULL_ASSERT aktiviert werden"
  #endif

  typedef enum {
    ASSERT_NO_ERROR =0,               // kein Fehler
    ASSERT_ERROR                      // Fehler
  }SYSCHECK_ASSERT_STATUS_t;

  typedef struct {
    SYSCHECK_ASSERT_STATUS_t status;  // status ob fehler oder nicht
    uint32_t err_line;                // Zeilen-Nummer vom Fehler
    uint8_t *err_file;                // Filename vom Fehler
  }SYSCHECK_ASSERT_t;
  SYSCHECK_ASSERT_t SYSCHECK_ASSERT;
#endif




#if  SYSCHECK_USE_ASSERT==1
//--------------------------------------------------------------
// Pointer fr den Assert-Mode
//--------------------------------------------------------------
  ADC_Common_TypeDef  * ADC_DBG;
  ADC_TypeDef         * ADC1_DBG;
  ADC_TypeDef         * ADC2_DBG;
  ADC_TypeDef         * ADC3_DBG;
  CAN_TypeDef         * CAN1_DBG;
  CAN_TypeDef         * CAN2_DBG;
  CRC_TypeDef         * CRC_DBG;
  CRYP_TypeDef        * CRYP_DBG;
  DAC_TypeDef         * DAC_DBG;
  DBGMCU_TypeDef      * DBGMCU_DBG;
  DCMI_TypeDef        * DCMI_DBG;
  DMA_TypeDef         * DMA1_DBG;
  DMA_Stream_TypeDef  * DMA1_Stream0_DBG;
  DMA_Stream_TypeDef  * DMA1_Stream1_DBG;
  DMA_Stream_TypeDef  * DMA1_Stream2_DBG;
  DMA_Stream_TypeDef  * DMA1_Stream3_DBG;
  DMA_Stream_TypeDef  * DMA1_Stream4_DBG;
  DMA_Stream_TypeDef  * DMA1_Stream5_DBG;
  DMA_Stream_TypeDef  * DMA1_Stream6_DBG;
  DMA_Stream_TypeDef  * DMA1_Stream7_DBG;
  DMA_TypeDef         * DMA2_DBG;
  DMA_Stream_TypeDef  * DMA2_Stream0_DBG;
  DMA_Stream_TypeDef  * DMA2_Stream1_DBG;
  DMA_Stream_TypeDef  * DMA2_Stream2_DBG;
  DMA_Stream_TypeDef  * DMA2_Stream3_DBG;
  DMA_Stream_TypeDef  * DMA2_Stream4_DBG;
  DMA_Stream_TypeDef  * DMA2_Stream5_DBG;
  DMA_Stream_TypeDef  * DMA2_Stream6_DBG;
  DMA_Stream_TypeDef  * DMA2_Stream7_DBG;
  EXTI_TypeDef        * EXTI_DBG;
  ETH_TypeDef         * ETH_DBG;
  FLASH_TypeDef       * FLASH_DBG;
  FSMC_Bank1_TypeDef  * FSMC_Bank1_DBG;
  FSMC_Bank1E_TypeDef * FSMC_Bank1E_DBG;
  FSMC_Bank2_TypeDef  * FSMC_Bank2_DBG;
  FSMC_Bank3_TypeDef  * FSMC_Bank3_DBG;
  FSMC_Bank4_TypeDef  * FSMC_Bank4_DBG;
  GPIO_TypeDef        * GPIOA_DBG;
  GPIO_TypeDef        * GPIOB_DBG;
  GPIO_TypeDef        * GPIOC_DBG;
  GPIO_TypeDef        * GPIOD_DBG;
  GPIO_TypeDef        * GPIOE_DBG;
  GPIO_TypeDef        * GPIOF_DBG;
  GPIO_TypeDef        * GPIOG_DBG;
  GPIO_TypeDef        * GPIOH_DBG;
  GPIO_TypeDef        * GPIOI_DBG;
  HASH_TypeDef        * HASH_DBG;
  I2C_TypeDef         * I2C1_DBG;
  I2C_TypeDef         * I2C2_DBG;
  I2C_TypeDef         * I2C3_DBG;
  IWDG_TypeDef        * IWDG_DBG;
  PWR_TypeDef         * PWR_DBG;
  RCC_TypeDef         * RCC_DBG;
  RNG_TypeDef         * RNG_DBG;
  RTC_TypeDef         * RTC_DBG;
  SDIO_TypeDef        * SDIO_DBG;
  SPI_TypeDef         * SPI1_DBG;
  SPI_TypeDef         * SPI2_DBG;
  SPI_TypeDef         * SPI3_DBG;
  SYSCFG_TypeDef      * SYSCFG_DBG;
  TIM_TypeDef         * TIM1_DBG;
  TIM_TypeDef         * TIM2_DBG;
  TIM_TypeDef         * TIM3_DBG;
  TIM_TypeDef         * TIM4_DBG;
  TIM_TypeDef         * TIM5_DBG;
  TIM_TypeDef         * TIM6_DBG;
  TIM_TypeDef         * TIM7_DBG;
  TIM_TypeDef         * TIM8_DBG;
  TIM_TypeDef         * TIM9_DBG;
  TIM_TypeDef         * TIM10_DBG;
  TIM_TypeDef         * TIM11_DBG;
  TIM_TypeDef         * TIM12_DBG;
  TIM_TypeDef         * TIM13_DBG;
  TIM_TypeDef         * TIM14_DBG;
  USART_TypeDef       * USART1_DBG;
  USART_TypeDef       * USART2_DBG;
  USART_TypeDef       * USART3_DBG;
  USART_TypeDef       * UART4_DBG;
  USART_TypeDef       * UART5_DBG;
  USART_TypeDef       * USART6_DBG;
  WWDG_TypeDef        * WWDG_DBG;
#endif


//--------------------------------------------------------------
// Enums und Strukturen fr "UB_SysCheck_MCO"
//--------------------------------------------------------------
#if SYSCHECK_USE_MCO==1
  typedef enum {
    MCO_DIV1 =0,      // vorteiler = 1
    MCO_DIV2,         // vorteiler = 2
    MCO_DIV3,         // vorteiler = 3
    MCO_DIV4,         // vorteiler = 4
    MCO_DIV5          // vorteiler = 5
  }MCO_PRESCALER;

  typedef enum {
    MCO1_LSE =0,      // LSE (external Quarz Lo-Speed)
    MCO1_HSE,         // HSE (external Quarz Hi-Speed)
    MCO1_HSI,         // HSI (internal RC)
    MCO1_PLLCLK       // PLL-Clock
  }MCO1_SOURCE;

  typedef enum {
    MCO2_SYSCLOCK =0, // Sysclock
    MCO2_PLLI2SCLK,   // I2S-Clock
    MCO2_HSE,         // HSE (external Quarz Hi-Speed)
    MCO2_PLLCLK       // PLL-Clock
  }MCO2_SOURCE;

  // MCO1 an PA8
  #define  MCO1_GPIO_CLOCK     RCC_AHB1Periph_GPIOA
  #define  MCO1_GPIO_PORT      GPIOA
  #define  MCO1_GPIO_PIN       GPIO_Pin_8
  #define  MCO1_GPIO_SOURCE    GPIO_PinSource8

  // MCO2 an PC9
  #define  MCO2_GPIO_CLOCK     RCC_AHB1Periph_GPIOC
  #define  MCO2_GPIO_PORT      GPIOC
  #define  MCO2_GPIO_PIN       GPIO_Pin_9
  #define  MCO2_GPIO_SOURCE    GPIO_PinSource9
#endif




//--------------------------------------------------------------
// Globale Funktionen
//--------------------------------------------------------------
#if  SYSCHECK_USE_CLOCK==1
SYSCHECK_t UB_SysCheck_Clock(void);
#endif

#if  SYSCHECK_USE_ASSERT==1
void UB_SysCheck_Assert_Init(void);
SYSCHECK_t UB_SysCheck_Assert(void);
#endif

#if SYSCHECK_USE_MCO==1
void UB_SysCheck_MCO1(MCO1_SOURCE source, MCO_PRESCALER prescaler);
void UB_SysCheck_MCO2(MCO2_SOURCE source, MCO_PRESCALER prescaler);
#endif


//--------------------------------------------------------------
#endif // __STM32F4_UB_SYSCHECK_H
